# Formatting configuration for locale eo

languages={'gu': u'gu\u011darata', 'gd': 'gaela', 'ga': 'irlanda', 'gn': 'gvarania', 'gl': 'galega', 'la': 'latino', 'ln': 'lingala', 'lo': u'la\u016da', 'tt': 'tatara', 'tr': 'turka', 'ts': 'conga', 'lv': 'latva', 'to': 'tongaa', 'lt': 'litova', 'tk': 'turkmena', 'th': 'taja', 'ti': 'tigraja', 'tg': u'ta\u011dika', 'te': 'telugua', 'ta': 'tamila', 'yi': 'jida', 'yo': 'joruba', 'de': 'germana', 'da': 'dana', 'dz': 'dzonko', 'st': 'sota', 'qu': u'ke\u0109ua', 'el': 'greka', 'eo': 'esperanto', 'en': 'angla', 'zh': u'\u0109ina', 'za': u'\u011duanga', 'uk': 'ukraina', 'eu': u'e\u016dska', 'et': 'estona', 'es': 'hispana', 'ru': 'rusa', 'rw': 'ruanda', 'rm': u'roman\u0109a', 'rn': 'burunda', 'ro': 'rumana', 'bn': 'bengala', 'be': 'belorusa', 'bg': 'bulgara', 'ba': u'ba\u015dkira', 'wo': 'volofa', 'jv': 'java', 'bo': 'tibeta', 'bh': 'bihara', 'bi': 'bislamo', 'br': 'bretona', 'ja': 'japana', 'om': 'oroma', 'oc': 'okcitana', 'tw': 'akana', 'or': 'orijo', 'xh': 'ksosa', 'co': 'korsika', 'ca': 'kataluna', 'cy': 'kimra', 'cs': u'\u0109e\u0125a', 'ps': u'pa\u015dtua', 'pt': 'portugala', 'tl': 'filipina', 'pa': u'pan\u011daba', 'vi': 'vjetnama', 'pl': 'pola', 'hy': 'armena', 'hr': 'kroata', 'iu': 'inuita', 'hu': 'hungara', 'hi': 'hinda', 'ha': u'ha\u016dsa', 'he': 'hebrea', 'mg': 'malagasa', 'uz': 'uzbeka', 'ml': 'malajalama', 'mn': 'mongola', 'mi': 'maoria', 'ik': 'eskima', 'mk': 'makedona', 'ur': 'urduo', 'mt': 'malta', 'ms': 'malaja', 'mr': 'marata', 'ug': 'ujgura', 'my': 'birma', 'aa': 'afara', 'ab': u'ab\u0125aza', 'ss': 'svazia', 'af': 'afrikansa', 'tn': 'cvana', 'sw': 'svahila', 'is': 'islanda', 'am': 'amhara', 'it': 'itala', 'sv': 'sveda', 'ia': 'interlingvao', 'as': 'asama', 'ar': 'araba', 'su': 'sunda', 'zu': 'zulua', 'ay': 'ajmara', 'az': u'azerbaj\u011dana', 'ie': 'okcidentalo', 'id': 'indonezia', 'sk': 'slovaka', 'nl': 'nederlanda', 'no': 'norvega', 'na': 'naura', 'ne': 'nepala', 'vo': 'volapuko', 'fr': 'franca', 'sm': 'samoa', 'fy': 'frisa', 'fa': 'persa', 'fi': 'finna', 'fj': u'fi\u011dia', 'sa': 'sanskrito', 'fo': 'feroa', 'ka': 'kartvela', 'kk': u'kaza\u0125a', 'sr': 'serba', 'sq': 'albana', 'ko': 'korea', 'kn': 'kanara', 'km': 'kmera', 'kl': 'gronlanda', 'ks': u'ka\u015dmira', 'si': 'sinhala', 'sh': 'serbo-Kroata', 'so': 'somala', 'sn': u'\u015dona', 'ku': 'kurda', 'sl': 'slovena', 'ky': 'kirgiza', 'sg': 'sangoa', 'sd': 'sinda'}

countries={'BD': u'Banglade\u015do', 'BE': 'Belgujo', 'BF': 'Burkino', 'BG': 'Bulgarujo', 'BA': 'Bosnio-Hercegovino', 'BB': 'Barbado', 'WF': 'Valiso kaj Futuno', 'BM': 'Bermudoj', 'BN': 'Brunejo', 'BO': 'Bolivio', 'BH': 'Barejno', 'BI': 'Burundo', 'BJ': 'Benino', 'BT': 'Butano', 'JM': 'Jamajko', 'BW': 'Bocvano', 'WS': 'Samoo', 'BR': 'Brazilo', 'BS': 'Bahamoj', 'BY': 'Belorusujo', 'BZ': 'Belizo', 'RU': 'Rusujo', 'RW': 'Ruando', 'RE': 'Reunio', 'TM': 'Turkmenujo', 'TJ': u'Ta\u011dikujo', 'RO': 'Rumanujo', 'GW': u'Gvineo-Bisa\u016do', 'GU': 'Gvamo', 'GT': 'Gvatemalo', 'GS': u'Sud-Georgio kaj Sud-Sandvi\u0109insuloj', 'GR': 'Grekujo', 'GQ': 'Ekvatora Gvineo', 'GP': 'Gvadelupo', 'JP': 'Japanujo', 'GY': 'Gujano', 'GF': 'Franca Gviano', 'GE': 'Kartvelujo', 'GD': 'Grenado', 'GB': u'Unui\u011dinta Re\u011dlando', 'GA': 'Gabono', 'SV': 'Salvadoro', 'GN': 'Gvineo', 'GM': 'Gambio', 'GL': 'Gronlando', 'GI': u'\u011cibraltaro', 'GH': 'Ganao', 'OM': 'Omano', 'TN': 'Tunizio', 'JO': 'Jordanio', 'SP': 'Serbujo', 'HR': 'Kroatujo', 'HT': 'Haitio', 'HU': 'Hungarujo', 'HN': 'Honduro', 'HM': 'Herda kaj Makdonaldaj Insuloj', 'VE': 'Venezuelo', 'PR': 'Puerto-Riko', 'PW': u'Bela\u016do', 'PT': 'Portugalujo', 'SJ': 'Svalbardo kaj Jan-Majen-insulo', 'PY': 'Paragvajo', 'IQ': 'Irako', 'PA': 'Panamo', 'PF': 'Franca Polinezio', 'PG': 'Papuo-Nov-Gvineo', 'PE': 'Peruo', 'PK': 'Pakistano', 'PH': 'Filipinoj', 'PN': 'Pitkarna Insulo', 'PL': 'Pollando', 'PM': 'Sent-Piero kaj Mikelono', 'ZM': 'Zambio', 'EH': 'Okcidenta Saharo', 'EE': 'Estonujo', 'EG': 'Egipto', 'ZA': 'Sud-Afriko', 'EC': 'Ekvadoro', 'IT': 'Italujo', 'VN': 'Vjetnamo', 'SB': 'Salomonoj', 'ET': 'Etiopujo', 'SO': 'Somalujo', 'ZW': 'Zimbabvo', 'SA': u'Sa\u016dda Arabujo', 'ES': 'Hispanujo', 'ER': 'Eritreo', 'MD': 'Moldavujo', 'MG': 'Madagaskaro', 'MA': 'Maroko', 'MC': 'Monako', 'UZ': 'Uzbekujo', 'MM': 'Mjanmao', 'ML': 'Malio', 'MN': 'Mongolujo', 'MH': u'Mar\u015daloj', 'MK': 'Makedonujo', 'MU': u'Ma\u016dricio', 'MT': 'Malto', 'MW': 'Malavio', 'MV': 'Maldivoj', 'MQ': 'Martiniko', 'MP': 'Nord-Marianoj', 'MR': u'Ma\u016dritanujo', 'UG': 'Ugando', 'MY': 'Malajzio', 'MX': 'Meksiko', 'IL': 'Israelo', 'FR': 'Francujo', 'IO': 'Brita Hindoceana Teritorio', 'SH': 'Sent-Heleno', 'FI': 'Finnlando', 'FJ': u'Fi\u011doj', 'FM': 'Mikronezio', 'FO': 'Ferooj', 'NI': 'Nikaragvo', 'NL': 'Nederlando', 'NO': 'Norvegujo', 'NA': 'Namibio', 'VU': 'Vanuatuo', 'NC': 'Nov-Kaledonio', 'NE': u'Ni\u011dero', 'NF': 'Norfolkinsulo', 'NG': u'Ni\u011derio', 'NZ': 'Nov-Zelando', 'NP': 'Nepalo', 'NR': 'Nauro', 'NU': 'Niuo', 'CK': 'Kukinsuloj', 'CI': 'Ebur-Bordo', 'CH': 'Svisujo', 'CO': 'Kolombio', 'CN': u'\u0108inujo', 'CM': 'Kameruno', 'CL': u'\u0108ilio', 'CA': 'Kanado', 'CG': 'Kongolo', 'CF': 'Centr-Afrika Respubliko', 'CZ': u'\u0108e\u0125ujo', 'CY': 'Kipro', 'CR': 'Kostariko', 'CV': 'Kabo-Verdo', 'CU': 'Kubo', 'SZ': 'Svazilando', 'SY': 'Sirio', 'KG': 'Kirgizistano', 'KE': 'Kenjo', 'SR': 'Surinamo', 'KI': 'Kiribato', 'KH': u'Kambo\u011do', 'KN': 'Sent-Kristofo kaj Neviso', 'KM': 'Komoroj', 'ST': 'Sao-Tomeo kaj Principeo', 'SK': 'Slovakujo', 'KR': 'Sud-Koreo', 'SI': 'Slovenujo', 'KP': 'Nord-Koreo', 'KW': 'Kuvajto', 'SN': 'Senegalo', 'SM': 'San-Marino', 'SL': 'Siera-Leono', 'SC': u'Sej\u015deloj', 'KZ': u'Kaza\u0125stano', 'KY': 'Kejmanoj', 'SG': 'Singapuro', 'SE': 'Svedujo', 'SD': 'Sudano', 'DO': 'Domingo', 'DM': 'Dominiko', 'DJ': u'\u011cibutio', 'DK': 'Danujo', 'VG': 'Britaj Virgulininsuloj', 'DE': 'Germanujo', 'YE': 'Jemeno', 'DZ': u'Al\u011derio', 'US': 'Usono', 'UY': 'Urugvajo', 'YT': 'Majoto', 'UM': 'Usonaj malgrandaj insuloj', 'LB': 'Libano', 'LC': 'Sent-Lucio', 'LA': 'Laoso', 'TV': 'Tuvalo', 'TW': 'Tajvano', 'TT': 'Trinidado kaj Tobago', 'TR': 'Turkujo', 'LK': 'Sri-Lanko', 'LI': u'Li\u0125ten\u015dtejno', 'LV': 'Latvujo', 'TO': 'Tongo', 'LT': 'Litovujo', 'LU': 'Luksemburgo', 'LR': 'Liberio', 'LS': 'Lesoto', 'TH': 'Tajlando', 'TG': 'Togolo', 'TD': u'\u0108ado', 'LY': 'Libio', 'VA': 'Vatikano', 'VC': 'Sent-Vincento kaj la Grenadinoj', 'AE': u'Unui\u011dintaj Arabaj Emirlandos', 'AD': 'Andoro', 'AG': 'Antigvo-Barbudo', 'AF': 'Afganujo', 'AI': 'Angvilo', 'VI': 'Usonaj Virgulininsuloj', 'IS': 'Islando', 'IR': 'Irano', 'AM': 'Armenujo', 'AL': 'Albanujo', 'AO': 'Angolo', 'AN': 'Nederlandaj Antiloj', 'AQ': 'Antarkto', 'AR': 'Argentino', 'AU': u'A\u016dstralio', 'AT': u'A\u016dstrujo', 'AW': 'Arubo', 'IN': 'Hindujo', 'TZ': 'Tanzanio', 'AZ': u'Azerbaj\u011dano', 'IE': 'Irlando', 'ID': 'Indonezio', 'UA': 'Ukrajno', 'QA': 'Kataro', 'MZ': 'Mozambiko'}

months=['januaro', 'februaro', 'marto', 'aprilo', 'majo', 'junio', 'julio', u'a\u016dgusto', 'septembro', 'oktobro', 'novembro', 'decembro']

abbrMonths=['jan', 'feb', 'mar', 'apr', 'maj', 'jun', 'jul', u'a\u016dg', 'sep', 'okt', 'nov', 'dec']

days=['lundo', 'mardo', 'merkredo', u'\u0135a\u016ddo', 'vendredo', 'sabato', u'diman\u0109o']

abbrDays=['lu', 'ma', 'me', u'\u0135a', 've', 'sa', 'di']

dateFormats={'medium': '%Y-%%(abbrmonthname)s-%d', 'full': '%%(dayname)s, %d-a de %%(monthname)s %Y', 'long': '%Y-%%(monthname)s-%d', 'short': '%y-%m-%d'}

numericSymbols={'group': u'\xa0', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}