body { margin:0;padding:0;
      font-family:verdana,sans-serif;
      font-size:12px; 
       margin-bottom:50px;
      }
#top_background{
          background-image:url('/tg_static/images/top_bg.png');
          background-repeat:repeat-x;
      }
#top {
        height:100px;
        width:100%;
        background-image:url('/tg_static/images/top.png');
        background-repeat:no-repeat;
        background-position:top right;
     }
#top h1 { font-size:22px;color:#fff;font-weight:100;margin:0;padding:0;
          margin-left:20px;padding-top:8px;}
#top h1 a {color:#fff}
h1,h2,h3,h4,h5 {color:#1F76BD}
h2 {margin-top:0;padding-top:0}
#main_content {text-align:left;width:750px}
#main_content a {color:#1F76BD;}
#main_content #tools { border-top:1px solid #1F76BD}
#main_content #tools h4 { padding-bottom:3px;margin-bottom:3px}
#left_col { width:229px;}
#left_col a {color:#1F76BD;}
button { background-color:#1F76BD;
         color:#fff;
         margin-left:4px;
         text-align:center;
       }
#under_the_hood
{
    width:100%;
    text-align:center;
    position:absolute;bottom:0px;
}

.disabled, .disabled h4 {
    color: grey !important;
}

dl.param_list dd {
    float: left;
    width: 110px;
    text-align: right;
    font-weight: bold;
}
dl.param_list dt {
    margin-left: 160px;
}


