# Formatting configuration for locale sw_KE

languages={'sw': 'Kiswahili'}

countries={'DO': 'Jamhuri ya Dominikan', 'FR': 'Ufaransa', 'DM': 'Dominika', 'DJ': 'Jibuti', 'BA': 'Bosnia na Herzegowina', 'BE': 'Ubelgiji', 'HR': 'Kroatia', 'DE': 'Udachi', 'JP': 'Ujapani', 'VC': 'Saint Vincent na Grenadines', 'HU': 'Hungaria', 'JM': 'Jamaika', 'ZA': 'Afrika ya Kusini', 'MH': 'Visiwa vya Marshall', 'US': 'Muungano wa Nchi za Amerika', 'BR': 'Brazili', 'BS': 'Visiwa vya Bahama', 'CO': 'Kolombia', 'UY': 'Urugwai', 'DK': 'Udenmarki', 'IS': 'Barafu', 'NI': 'Nikaragua', 'NL': 'Uholanzi', 'CM': 'Kamerun', 'PT': 'Ureno', 'NO': 'Unorwe', 'TT': 'Trinidad na Tobago', 'PY': 'Paragwai', 'TR': 'Uturuki', 'TL': 'Timor ya Mashariki', 'LU': 'Luksemburg', 'PG': 'Papua Guinea Mpya', 'NP': 'Nepali', 'PH': 'Filipino', 'GQ': 'Guinea ya Ikweta', 'EG': 'Misri', 'VA': 'Vatikano', 'CI': 'Pwani ya Pembe', 'CH': 'Uswisi', 'AE': 'Muugano wa Falme za Nchi za Kiarabu', 'CN': 'Uchina', 'AG': 'Antigua na Barbuda', 'RU': 'Urusi', 'IQ': 'Iraki', 'YE': 'Yemeni', 'CA': 'Kanada', 'IR': 'Uajemi', 'CG': 'Kongo', 'CF': 'Jamhuri ya Afrika ya Kati', 'EC': 'Ekvado', 'IT': 'Uitaliani', 'CZ': 'Jamhuri ya Czech', 'AR': 'Ajentina', 'GB': 'Uingereza', 'IN': 'Uhindi', 'ET': 'Uhabeshi', 'CV': 'Rasi Verde', 'ES': 'Uhispania', 'MA': 'Moroko', 'MC': 'Monako', 'KE': 'Kenya', 'KH': 'Kampuchea', 'KN': 'Saint Kitts na Nevis', 'KM': 'Visiwa vya Komoro', 'ST': 'Sao Tome na Principe', 'KR': 'Korea ya Kusini', 'TD': 'Chadi', 'KP': 'Korea ya Kaskazini', 'SO': 'Somali', 'TZ': 'Tanzania', 'SC': 'Visiwa vya Shelisheli', 'SB': 'Visiwa vya Solomon', 'SA': 'Arabuni Saudi', 'MZ': 'Msumbiji', 'MY': 'Malasya', 'MX': 'Meksiko', 'SE': 'Uswidi', 'IL': 'Uyahudi'}

months=['Januari', 'Februari', 'Machi', 'Aprili', 'Mei', 'Juni', 'Julai', 'Agosti', 'Septemba', 'Oktoba', 'Novemba', 'Desemba']

abbrMonths=['Jan', 'Feb', 'Mar', 'Apr', 'Mei', 'Jun', 'Jul', 'Ago', 'Sep', 'Okt', 'Nov', 'Des']

days=['Jumatatu', 'Jumanne', 'Jumatano', 'Alhamisi', 'Ijumaa', 'Jumamosi', 'Jumapili']

abbrDays=['Jtt', 'Jnn', 'Jtn', 'Alh', 'Iju', 'Jmo', 'Jpi']

dateFormats={'medium': '%Y %%(abbrmonthname)s %d ', 'full': '%%(dayname)s, %Y %%(monthname)s %d ', 'long': '%Y %%(monthname)s %d ', 'short': '%y/%m/%d'}

numericSymbols={'group': ',', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': '.', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}