# Formatting configuration for locale so_SO

languages={'so': 'Soomaali'}

countries={'BD': 'Bangaala-Deesh', 'BE': 'Beljiyam', 'BA': 'Boosniya Heersigoviina', 'BB': 'Baarbadoos', 'BH': 'Baxrayn', 'BJ': 'Beniin', 'JM': 'Jameyka', 'JO': 'Urdun', 'BR': 'Braasiil', 'RU': 'Ruush', 'RO': 'Rumaaniya', 'GR': 'Giriigga', 'JP': 'Jabbaan', 'GD': 'Giriinaada', 'GN': 'Gini', 'OM': 'Cumaan', 'TN': 'Tuniisiya', 'IL': "Israa'iil", 'HR': 'Korweeshiya', 'HU': 'Hangeri', 'PT': 'Bortuqaal', 'PK': 'Bakistaan', 'PH': 'Filibiin', 'PL': 'Booland', 'ZM': 'Saambiya', 'EG': 'Masar', 'ZA': 'Koonfur Afrika', 'VN': 'Fiyetnaam', 'KZ': 'Kasaakhistaan', 'ET': 'Itoobiya', 'ZW': 'Simbaabwe', 'ES': 'Isbeyn', 'MA': 'Marooko', 'MC': 'Moonako', 'ML': 'Maali', 'MK': 'Makadooniya', 'MT': 'Maalda', 'MW': 'Malaawi', 'MV': 'Maaldiqeen', 'MR': 'Muritaaniya', 'UG': 'Ugaanda', 'MX': 'Meksiko', 'MZ': 'Musambiig', 'FR': 'Faransiis', 'FI': 'Fiinlaand', 'NI': 'Nikaraaguwa', 'NO': 'Noorweey', 'NA': 'Namiibiya', 'NG': 'Nayjeeriya', 'NZ': 'Neyuusilaand', 'CH': 'Swiiserlaand', 'CN': 'Shiinaha', 'CM': 'Kameruun', 'CL': 'Jili', 'CA': 'Kanada', 'CU': 'Kuuba', 'SY': 'Suuriya', 'KE': 'Kiiniya', 'KH': 'Kamboodiya', 'KR': 'Kuuriyada Koonfureed', 'KP': 'Kuuriyada Waqooyi', 'SO': 'Soomaaliya', 'SL': 'Siraaliyoon', 'KW': 'Kuwayt', 'SA': 'Sacuudi Carabiya', 'SE': 'Iswidhan', 'SD': 'Sudaan', 'DJ': 'Jabuuti', 'DK': 'Danmaark', 'DE': 'Jarmal', 'YE': 'Yaman', 'US': 'Qaramada Midoobey ee Maraykanka', 'LB': 'Lubnaan', 'TR': 'Turki', 'LK': 'Siirilaanka', 'LV': 'Laatfiya', 'LU': 'Luksemboorg', 'LR': 'Laybeeriya', 'LS': 'Losooto', 'TH': 'Taylaand', 'TG': 'Toogo', 'TD': 'Jaad', 'LY': 'Liibiya', 'VA': 'Faatikaan', 'AE': 'Imaaraadka Carabta ee Midoobay', 'VE': 'Fenisuweela', 'AF': 'Afgaanistaan', 'IQ': 'Ciraaq', 'IS': 'Iislaand', 'IR': 'Iiraan', 'AM': 'Armeeniya', 'IT': 'Talyaani', 'AO': 'Angoola', 'AU': 'Awstraaliya', 'AT': 'Osteeriya', 'IN': 'Hindiya', 'TZ': 'Tansaaniya', 'IE': 'Ayrlaanda', 'ID': 'Indoneesiya', 'QA': 'Qadar'}

months=['Bisha Koobaad', 'Bisha Labaad', 'Bisha Saddexaad', 'Bisha Afraad', 'Bisha Shanaad', 'Bisha Lixaad', 'Bisha Todobaad', 'Bisha Sideedaad', 'Bisha Sagaalaad', 'Bisha Tobnaad', 'Bisha Kow iyo Tobnaad', 'Bisha Laba iyo Tobnaad']

abbrMonths=['Kob', 'Lab', 'Sad', 'Afr', 'Sha', 'Lix', 'Tod', 'Sid', 'Sag', 'Tob', 'KIT', 'LIT']

days=['Isniin', 'Salaaso', 'Arbaco', 'Khamiis', 'Jimco', 'Sabti', 'Axad']

abbrDays=['Isn', 'Sal', 'Arb', 'Kha', 'Jim', 'Sab', 'Axa']

dateFormats={'medium': '%d-%%(abbrmonthname)s-%Y', 'full': '%%(dayname)s, %%(monthname)s %d, %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d/%m/%y'}

numericSymbols={'group': ',', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': '.', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}