.grid { background-color:#e3e3e3;font-size:12px;font-family:verdana,sans-serif;border:none;}
.grid td {padding:3px;border:none;}
.grid .action_cell { text-align:right; }
.grid THEAD { text-align:left;background-color:#f0f0f0;color:#666;}
.grid .heading img { float:right;margin-left:2px;margin-right:3px; } 
.grid .heading a { text-decoration:none;color:#333; }
.grid td a { text-decoration:none;color:#333}
.odd{background-color:#edf3fe}
.even{background-color:#fff}
.pointer {cursor:pointer}
.column_chooser_link {position:relative;background-color:#e3e3e3;}
.column_chooser_link ul {position:absolute;display:none;top:10px;right:-45px;}
.column_chooser_list a {width:200px;display:block;padding:3px;background-color:#e3e3e3;}
.column_chooser_list a:hover {background-color:#cdcdcd;}
.column_chooser_list { padding:0;margin:0;list-style:none;background-color:#e3e3e3; }

