# Formatting configuration for locale om_KE

languages={'om': 'Oromoo'}

countries={'ET': 'Itoophiyaa', 'KE': 'Keeniyaa'}

months=['Amajjii', 'Guraandhala', 'Bitooteessa', 'Elba', 'Caamsa', 'Waxabajjii', 'Adooleessa', 'Hagayya', 'Fuulbana', 'Onkololeessa', 'Sadaasa', 'Muddee']

abbrMonths=['Ama', 'Gur', 'Bit', 'Elb', 'Cam', 'Wax', 'Ado', 'Hag', 'Ful', 'Onk', 'Sad', 'Mud']

days=['Wiixata', 'Qibxata', 'Roobii', 'Kamiisa', 'Jimaata', 'Sanbata', 'Dilbata']

abbrDays=['Wix', 'Qib', 'Rob', 'Kam', 'Jim', 'San', 'Dil']

dateFormats={'medium': '%d-%%(abbrmonthname)s-%y', 'full': '%%(dayname)s, %%(monthname)s %d, %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d/%m/%y'}

numericSymbols={'group': ',', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': '.', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}