# Formatting configuration for locale is_IS

languages={'is': u'\xedslenska'}

countries={'IS': u'\xcdsland'}

months=[u'jan\xfaar', u'febr\xfaar', 'mars', u'apr\xedl', u'ma\xed', u'j\xfan\xed', u'j\xfal\xed', u'\xe1g\xfast', 'september', u'okt\xf3ber', u'n\xf3vember', 'desember']

abbrMonths=['jan', 'feb', 'mar', 'apr', u'ma\xed', u'j\xfan', u'j\xfal', u'\xe1g\xfa', 'sep', 'okt', u'n\xf3v', 'des']

days=[u'm\xe1nudagur', u'\xferi\xf0judagur', u'mi\xf0vikudagur', 'fimmtudagur', u'f\xf6studagur', 'laugardagur', 'sunnudagur']

abbrDays=[u'm\xe1n', u'\xferi', u'mi\xf0', 'fim', u'f\xf6s', 'lau', 'sun']

dateFormats={'medium': '%d.%m.%Y', 'full': '%%(dayname)s, %d. %%(monthname)s %Y', 'long': '%d. %%(monthname)s %Y', 'short': '%d.%m.%Y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}