# Formatting configuration for locale ca_ES

languages={'gu': 'gujarati', 'gd': u'escoc\xe8s', 'ga': u'irland\xe8s', 'gn': u'guaran\xed', 'gl': 'gallec', 'la': u'llat\xed', 'ln': 'lingala', 'lo': u'laosi\xe0', 'tt': u't\xe0tar', 'tr': 'turc', 'ts': 'tsonga', 'lv': u'let\xf3', 'to': 'tonga', 'lt': u'litu\xe0', 'tk': 'turcman', 'th': 'thai', 'ti': 'tigrinya', 'tg': 'tadjik', 'te': 'telugu', 'ta': u't\xe0mil', 'yi': 'jiddish', 'yo': 'ioruba', 'de': 'alemany', 'da': u'dan\xe8s', 'dz': u'bhutan\xe8s', 'st': 'sotho', 'qu': u'qu\xe8txua', 'el': 'grec', 'eo': 'esperanto', 'en': u'angl\xe8s', 'zh': u'xin\xe9s', 'za': 'zhuang', 'uk': u'ucra\xefn\xe8s', 'eu': 'basc', 'et': u'estoni\xe0', 'es': 'espanyol', 'ru': 'rus', 'rw': 'kinyarwanda', 'rm': u'retorom\xe0nic', 'rn': 'kirundi', 'ro': u'roman\xe8s', 'bn': u'bengal\xed', 'be': u'bielor\xfas', 'bg': u'b\xfalgar', 'ba': 'baixkir', 'wo': u'w\xf2lof', 'jv': u'javan\xe8s', 'bo': u'tibet\xe0', 'bh': 'bihari', 'bi': 'bislama', 'br': u'bret\xf3', 'ja': u'japon\xe8s', 'om': 'oromo (afan)', 'oc': u'occit\xe0', 'tw': 'twi', 'or': 'oriya', 'xh': 'xosa', 'co': 'cors', 'ca': u'catal\xe0', 'cy': u'gal\xb7l\xe8s', 'cs': 'txec', 'ps': 'paixto', 'pt': u'portugu\xe8s', 'tl': u'tag\xe0log', 'pa': 'panjabi', 'vi': 'vietnamita', 'pl': u'polon\xe8s', 'hy': 'armeni', 'hr': 'croat', 'iu': 'inuktitut', 'hu': u'hongar\xe8s', 'hi': 'hindi', 'ha': 'hausa', 'he': 'hebreu', 'mg': 'malgaix', 'uz': 'uzbek', 'ml': 'malaialam', 'mo': 'moldau', 'mn': 'mongol', 'mi': 'maori', 'ik': 'inupiak', 'mk': 'macedoni', 'ur': u'urd\xfa', 'mt': u'malt\xe8s', 'ms': 'malai', 'mr': 'marathi', 'ug': 'uigur', 'my': u'birm\xe0', 'aa': u'\xe0far', 'ab': 'abkhaz', 'ss': 'siswati', 'af': 'afrikaans', 'tn': 'tswana', 'sw': 'swahili', 'is': u'island\xe8s', 'am': u'amh\xe0ric', 'it': u'itali\xe0', 'sv': 'suec', 'ia': 'interlingua', 'as': u'assam\xe8s', 'ar': u'\xe2rab', 'su': u'sundan\xe8s', 'zu': 'zulu', 'ay': 'aimara', 'az': u'\xe0zeri', 'ie': 'interlingue', 'id': 'indonesi', 'sk': 'eslovac', 'nl': u'neerland\xe8s', 'no': 'noruec', 'na': u'nauru\xe0', 'ne': u'nepal\xe8s', 'vo': 'volapuk', 'fr': u'franc\xe8s', 'sm': u'samo\xe0', 'fy': u'fris\xf3', 'fa': 'persa', 'fi': u'fin\xe8s', 'fj': u'fiji\xe0', 'sa': u's\xe0nscrit', 'fo': u'fero\xe8s', 'ka': u'georgi\xe0', 'kk': 'kazakh', 'sr': 'serbi', 'sq': u'alban\xe8s', 'ko': u'core\xe0', 'kn': 'kannada', 'km': u'cambodj\xe0', 'kl': u'greenland\xe8s', 'ks': 'caixmiri', 'si': u'sinhal\xe8s', 'sh': 'serbo-croat', 'so': 'somali', 'sn': 'shona', 'ku': 'kurd', 'sl': u'eslov\xe8', 'ky': u'kirgu\xeds', 'sg': 'sango', 'sd': 'sindhi'}

countries={'BD': 'Bangla Desh', 'BE': u'B\xe8lgica', 'BF': 'Burkina Faso', 'BG': u'Bulg\xe0ria', 'BA': u'B\xf2snia i Hercegovina', 'BB': 'Barbados', 'BM': 'Bermudes', 'BN': 'Brunei', 'BO': u'Bol\xedvia', 'BH': 'Bahrain', 'BI': 'Burundi', 'BJ': 'Benin', 'BT': 'Bhutan', 'JM': 'Jamaica', 'BW': 'Botswana', 'BR': 'Brasil', 'BS': 'Bahames', 'BY': u'Bielor\xfassia', 'BZ': 'Belize', 'RU': u'R\xfassia', 'RW': 'Rwanda', 'TL': 'Timor', 'TM': 'Turkmenistan', 'TJ': 'Tadjikistan', 'RO': 'Romania', 'TK': 'Tokelau', 'GW': 'Guinea Bissau', 'GT': 'Guatemala', 'GR': u'Gr\xe8cia', 'GQ': 'Guinea Equatorial', 'GP': 'Guadeloupe', 'JP': u'Jap\xf3', 'GY': 'Guyana', 'GF': 'Guaiana Francesa', 'GE': u'Ge\xf2rgia', 'GB': 'Regne Unit', 'GA': 'Gabon', 'GN': 'Guinea', 'GM': u'G\xe0mbia', 'GH': 'Ghana', 'OM': 'Oman', 'TN': u'Tun\xedsia', 'JO': u'Jord\xe0nia', 'SP': u'S\xe8rbia', 'HR': u'Cro\xe0cia', 'HT': u'Hait\xed', 'HU': 'Hongria', 'HK': 'Hong Kong', 'HN': 'Hondures', 'VE': u'Vene\xe7uela', 'PR': 'Puerto Rico', 'PT': 'Portugal', 'PY': 'Paraguai', 'IQ': 'Iraq', 'PA': u'Panam\xe0', 'PF': u'Polin\xe8sia Francesa', 'PG': 'Papua Nova Guinea', 'PE': u'Per\xfa', 'PK': 'Pakistan', 'PH': 'Filipines', 'PL': u'Pol\xf2nia', 'ZM': u'Z\xe0mbia', 'EH': u'S\xe0hara Occidental', 'EE': u'Est\xf2nia', 'EG': 'Egipte', 'ZA': u'Sud-\xe0frica', 'EC': 'Equador', 'IT': u'It\xe0lia', 'VN': 'Vietnam', 'ET': u'Eti\xf2pia', 'SO': u'Som\xe0lia', 'ZW': 'Zimbabwe', 'ES': 'Espanya', 'ER': 'Eritrea', 'MD': u'Mold\xe0via', 'MG': 'Madagascar', 'MA': 'Marroc', 'MC': u'M\xf2naco', 'UZ': 'Uzbekistan', 'MM': 'Myanmar', 'ML': 'Mali', 'MN': u'Mong\xf2lia', 'MK': u'Maced\xf2nia', 'MU': 'Maurici', 'MT': 'Malta', 'MQ': 'Martinica', 'MS': 'Montserrat', 'MR': u'Maurit\xe0nia', 'UG': 'Uganda', 'MY': u'Mal\xe0isia', 'MX': u'M\xe8xic', 'IL': 'Israel', 'FR': u'Fran\xe7a', 'FI': u'Finl\xe0ndia', 'FJ': 'Fiji', 'FM': u'Micron\xe8sia', 'NI': 'Nicaragua', 'NL': u'Pa\xefsos Baixos', 'NO': 'Noruega', 'NA': u'Nam\xedbia', 'VU': 'Vanuatu', 'NC': u'Nova Caled\xf2nia', 'NE': u'N\xedger', 'NG': u'Nig\xe8ria', 'NZ': 'Nova Zelanda', 'NP': 'Nepal', 'NU': 'Niue', 'CI': "Costa d'Ivori", 'CH': 'Switzerland', 'CO': u'Col\xf2mbia', 'CN': 'Xina', 'CM': 'Camerun', 'CL': 'Xile', 'CA': u'Canad\xe0', 'CG': 'Congo', 'CF': u'Rep\xfablica Centrafricana', 'CZ': u'Rep\xfablica Txeca', 'CY': 'Xipre', 'CR': 'Costa Rica', 'CV': 'Cap Verd', 'CU': 'Cuba', 'SZ': u'Swazil\xe0ndia', 'SY': u'S\xedria', 'KG': 'Kirgizistan', 'KE': 'Kenya', 'SR': 'Surinam', 'KI': 'Kiribati', 'KH': 'Cambodja', 'SV': 'El Salvador', 'KM': 'Comores', 'SK': u'Eslov\xe0quia', 'KR': 'Corea del Sud', 'SI': u'Eslov\xe8nia', 'KP': 'Corea del Nord', 'KW': 'Kuwait', 'SN': 'Senegal', 'SL': 'Sierra Leone', 'SC': 'Seychelles', 'KZ': 'Kazakhstan', 'SA': u'Ar\xe0bia Saud\xed', 'SG': 'Singapur', 'SE': u'Su\xe8cia', 'SD': 'Sudan', 'DO': u'Rep\xfablica Dominicana', 'DM': 'Dominica', 'DJ': 'Djibouti', 'DK': 'Dinamarca', 'VG': u'Illes Verges Brit\xe0niques', 'DE': 'Alemanya', 'YE': 'Iemen', 'DZ': u'Alg\xe8ria', 'US': 'Estats Units', 'UY': 'Uruguai', 'YU': u'Iugosl\xe0via', 'YT': 'Mayotte', 'LB': u'L\xedban', 'LA': 'Laos', 'TW': 'Taiwan', 'TT': 'Trinitat i Tobago', 'TR': 'Turquia', 'LK': 'Sri Lanka', 'LI': 'Liechtenstein', 'LV': u'Let\xf2nia', 'TO': 'Tonga', 'LT': u'Litu\xe0nia', 'LU': 'Luxemburg', 'LR': u'Lib\xe8ria', 'LS': 'Lesotho', 'TH': u'Tail\xe0ndia', 'TF': 'Territoris Meridionals Francesos', 'TG': 'Togo', 'TD': 'Txad', 'LY': u'L\xedbia', 'VA': u'Vatic\xe0', 'AE': u'Uni\xf3 dels Emirats \xc0rabs', 'AD': 'Andorra', 'AF': 'Afganistan', 'AI': 'Anguilla', 'VI': 'Illes Verges dels USA', 'IS': u'Isl\xe0ndia', 'IR': 'Iran', 'AM': u'Arm\xe8nia', 'AL': u'Alb\xe0nia', 'AO': 'Angola', 'AN': 'Antilles Holandeses', 'AR': 'Argentina', 'AU': u'Austr\xe0lia', 'AT': u'\xc2ustria', 'AW': 'Aruba', 'IN': u'\xcdndia', 'TZ': u'Tanz\xe0nia', 'AZ': 'Azerbaidjan', 'IE': 'Irlanda', 'ID': u'Indon\xe8sia', 'UA': u'Ucra\xefna', 'QA': 'Qatar', 'MZ': u'Mo\xe7ambic'}

months=['gener', 'febrer', u'mar\xe7', 'abril', 'maig', 'juny', 'juliol', 'agost', 'setembre', 'octubre', 'novembre', 'desembre']

abbrMonths=['gen.', 'feb.', u'mar\xe7', 'abr.', 'maig', 'juny', 'jul.', 'ag.', 'set.', 'oct.', 'nov.', 'des.']

days=['dilluns', 'dimarts', 'dimecres', 'dijous', 'divendres', 'dissabte', 'diumenge']

abbrDays=['dl.', 'dt.', 'dc.', 'dj.', 'dv.', 'ds.', 'dg.']

dateFormats={'medium': '%d/%m/%Y', 'full': '%%(dayname)s %d %%(monthname)s %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d/%m/%y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}