# Formatting configuration for locale pt_PT

languages={'lv': u'let\xe3o', 'ar': u'\xe1rabe', 'sl': 'esloveno', 'cs': 'checo', 'et': u'est\xf3nio', 'pl': 'polaco'}

countries={'BA': u'B\xf3snia-Herzegovina', 'YE': u'I\xe9men', 'BJ': 'Benim', 'HK': u'Hong Kong - Regi\xe3o Administrativa Especial da China', 'UM': 'Ilhas Minor Outlying (E.U.A)', 'GS': 'Ilhas South Georgia e South Sandwich', 'FK': 'Ilhas Falkland', 'YU': u'Jugosl\xe1via', 'BY': u'Bielorr\xfassia', 'PS': u'Territ\xf3rio Palestiniano', 'AZ': u'Azerbeij\xe3o', 'CM': u'Camar\xf5es', 'LA': u'Lao, Rep\xfablica Popular Democr\xe1tica', 'TW': u'Formosa, Prov\xedncia Chinesa', 'NC': u'Nova Caled\xf3nia', 'LV': u'Let\xf3nia', 'RE': 'Reunion', 'TM': u'Turquemenist\xe3o', 'TJ': u'Tajiquist\xe3o', 'PG': u'Papua Nova Guin\xe9', 'TF': u'Territ\xf3rios Franceses a Sul', 'TD': 'Tchade', 'RO': u'Rom\xe9nia', 'TC': 'Ilhas Turcas e Caicos', 'PL': u'Pol\xf3nia', 'GW': u'Guin\xe9-Bissau', 'VA': u'Santa S\xe9 (Estado da Cidade do Vaticano)', 'VC': 'Saint Vincent e Grenadines', 'AE': u'Emiratos \xc1rabes Unidos', 'EH': 'Sahara Ocidental', 'AG': u'Ant\xedgua e Barbuda', 'VG': u'Ilhas Virgin Brit\xe2nicas', 'EE': u'Est\xf3nia', 'VI': 'Ilhas Virgin E.U.A.', 'EG': 'Egipto', 'AM': u'Arm\xe9nia', 'VN': 'Vietname', 'AQ': u'Ant\xe1rctica', 'CZ': u'Rep\xfablica Checa', 'CX': 'Ilha do Natal', 'GL': u'Gronel\xe2ndia', 'ER': 'Eritreia', 'MD': u'Mold\xe1via, Rep\xfablica da', 'MG': u'Madag\xe1scar', 'KG': u'Quirguizist\xe3o', 'MC': u'M\xf3naco', 'KE': u'Qu\xe9nia', 'UZ': u'Uzbaquist\xe3o', 'MO': u'Macau - Regi\xe3o Administrativa Especial da China', 'KH': 'Camboja', 'KN': 'Saint Kitts e Nevis', 'MK': u'Maced\xf3nia, Rep\xfablica da', 'MU': u'Maur\xedcias', 'KR': 'Coreia do Sul', 'SI': u'Eslov\xe9nia', 'KP': 'Coreia do Norte', 'MP': 'Ilhas Mariana do Norte', 'SM': u'S\xe3o Marino', 'SC': 'Seicheles', 'KZ': u'Cazaquist\xe3o', 'KY': u'Ilhas Caim\xe3o', 'SG': 'Singapura'}

months=['janeiro', 'fevereiro', u'mar\xe7o', 'abril', 'maio', 'junho', 'julho', 'agosto', 'setembro', 'outubro', 'novembro', 'dezembro']

abbrMonths=['jan', 'fev', 'mar', 'abr', 'mai', 'jun', 'jul', 'ago', 'set', 'out', 'nov', 'dez']

days=['segunda-feira', u'ter\xe7a-feira', 'quarta-feira', 'quinta-feira', 'sexta-feira', u's\xe1bado', 'domingo']

abbrDays=['seg', 'ter', 'qua', 'qui', 'sex', u's\xe1b', 'dom']

dateFormats={'medium': '%Y/%m/%d', 'full': '%%(dayname)s, %d de %%(monthname)s de %Y', 'long': '%d de %%(monthname)s de %Y', 'short': '%y/%m/%d'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}