# Formatting configuration for locale nn

languages={'nb': u'norsk bokm\xe5l', 'nn': 'norsk nynorsk', 'no': 'norsk'}

countries={'NO': 'Noreg'}

months=['januar', 'februar', 'mars', 'april', 'mai', 'juni', 'juli', 'august', 'september', 'oktober', 'november', 'desember']

abbrMonths=['jan', 'feb', 'mar', 'apr', 'mai', 'jun', 'jul', 'aug', 'sep', 'okt', 'nov', 'des']

days=[u'm\xe5ndag', 'tysdag', 'onsdag', 'torsdag', 'fredag', 'laurdag', 'sundag']

abbrDays=[u'm\xe5', 'ty', 'on', 'to', 'fr', 'la', 'su']

dateFormats={'medium': '%d. %%(abbrmonthname)s. %Y', 'full': '%%(dayname)s %d. %%(monthname)s %Y', 'long': '%d. %%(monthname)s %Y', 'short': '%d.%m.%y'}

numericSymbols={'group': u'\xa0', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}