var RELATION_THICKNESS = 1;
var RELATION_OFFSET = 10;
var ROW_HEIGHT = 19;
var TABLE_WIDTH = 60;
var LETTER_WIDTH = 4;
var BUTTON_ENABLED = "#000";
var BUTTON_DISABLED = "#888";

var DESK_SIZE = 900;
var MAP_SIZE = 100;
